/*
 * cksumtest3.c
 *
 * Module test #3 for in_cksum()
 *
 * Copyright 2008-2009 Jaap Keuter and Greg Neujahr
 * Released into public domain.
 *
 * This module test goes at the actual CDP packet data. It tries the in_cksum(), the
 * expanded equivalent and cdp_checksum. Then it tries the cdp_checksum on packet
 * data with zeroed checksum (as the originator would have done).
 * Finally the results are duplicated by means of a modified cdp_checksum function,
 * but only for half the cases. ios_cdp_checksum tries to cover all cases.
 */

#include <glib.h>
#include "in_cksum.h"

/* An adaptation of real_cdp_checksum(), to match the all results */
unsigned short ios_cdp_checksum(const char *data, int length) {
        unsigned long sum = 0;
        const unsigned short *d = (const unsigned short *)data;
        unsigned char *ptr;
        short value; /* BUG: this is signed ! */

        while (length > 1) {
                sum += *d++;
                length -= 2;
        }
        if (length) {
                ptr = (unsigned char *)d;
                value = *ptr;
                value = ntohs(value);
                sum += value;
        }

        sum = (sum >> 16) + (sum & 0xffff);
        sum += (sum >> 16);
        sum = ~sum;
        sum = (unsigned short)sum;
        return sum;
}

/* An adaptation of real_cdp_checksum(), to match most results */
unsigned short actual_cdp_checksum(const char *data, int length) {
        unsigned long sum = -1; /* initialized to -1 */
        const unsigned short *d = (const unsigned short *)data;
        unsigned char *ptr;
        unsigned short value;

        while (length > 1) {
                sum += *d++;
                length -= 2;
        }
        if (length) {
                ptr = (unsigned char *)d;
                value = *ptr;
                value = ntohs(value); /* reads network order data into host order register */
                sum += value;
        }

        sum = (sum >> 16) + (sum & 0xffff);
        sum += (sum >> 16);
        sum = ~sum;
        sum = (unsigned short)sum;
        return sum;
}

/* A proposed checksum routine giving real live CDP checksums */
unsigned short real_cdp_checksum(const char *data, int length) {
        unsigned long sum = 0;
        const unsigned short *d = (const unsigned short *)data;
        unsigned char *ptr;
        unsigned short value;

        while (length > 1) {
                sum += *d++;
                length -= 2;
        }
        if (length) {
                ptr = (unsigned char *)d;
                value = *ptr;
                value = htons(value);
                sum += value;
        }

        sum = (sum >> 16) + (sum & 0xffff);
        sum += (sum >> 16);
        sum = ~sum;
        sum = (unsigned short)sum;
        return sum;
}

/* Wiresharks' CDP checksum routine verification, based on in_cksum() */
unsigned short wireshark_cdp_checksum(const char *data, int length) {
        unsigned long sum = 0;
        const unsigned short *d = (const unsigned short *)data;
        unsigned char *ptr;
        unsigned short value;

        while (length > 1) {
                sum += *d++;
                length -= 2;
        }
        if (length) {
                ptr = (unsigned char *)d;
                value = *ptr;
                sum += value;
        }

        sum = (sum >> 16) + (sum & 0xffff);
        sum += (sum >> 16);
        sum = ~sum;
        sum = (unsigned short)sum;
        return sum;
}

int main(int argc, char *argv[])
{
  int checksum;
  vec_t vector[2];
  const char dataset[] __attribute__ ((aligned)) = { /* CDP packet data with checksum intact */
    0x02, 0xb4, 0xdb, 0x37, 0x00, 0x01, 0x00, 0x1a, 0x73, 0x77, 0x69, 0x74, 0x63, 0x68, 0x66, 0x6f,
    0x78, 0x32, 0x2e, 0x66, 0x6f, 0x78, 0x70, 0x61, 0x77, 0x73, 0x2e, 0x6e, 0x65, 0x74, 0x00, 0x02,
    0x00, 0x11, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0xcc, 0x00, 0x04, 0x0a, 0x00, 0x00, 0xfe, 0x00,
    0x03, 0x00, 0x13, 0x46, 0x61, 0x73, 0x74, 0x45, 0x74, 0x68, 0x65, 0x72, 0x6e, 0x65, 0x74, 0x30,
    0x2f, 0x31, 0x00, 0x04, 0x00, 0x08, 0x00, 0x00, 0x00, 0x28, 0x00, 0x05, 0x00, 0xe1, 0x43, 0x69,
    0x73, 0x63, 0x6f, 0x20, 0x49, 0x6e, 0x74, 0x65, 0x72, 0x6e, 0x65, 0x74, 0x77, 0x6f, 0x72, 0x6b,
    0x20, 0x4f, 0x70, 0x65, 0x72, 0x61, 0x74, 0x69, 0x6e, 0x67, 0x20, 0x53, 0x79, 0x73, 0x74, 0x65,
    0x6d, 0x20, 0x53, 0x6f, 0x66, 0x74, 0x77, 0x61, 0x72, 0x65, 0x20, 0x0a, 0x49, 0x4f, 0x53, 0x20,
    0x28, 0x74, 0x6d, 0x29, 0x20, 0x43, 0x32, 0x39, 0x35, 0x30, 0x20, 0x53, 0x6f, 0x66, 0x74, 0x77,
    0x61, 0x72, 0x65, 0x20, 0x28, 0x43, 0x32, 0x39, 0x35, 0x30, 0x2d, 0x49, 0x36, 0x4b, 0x32, 0x4c,
    0x32, 0x51, 0x34, 0x2d, 0x4d, 0x29, 0x2c, 0x20, 0x56, 0x65, 0x72, 0x73, 0x69, 0x6f, 0x6e, 0x20,
    0x31, 0x32, 0x2e, 0x31, 0x28, 0x32, 0x32, 0x29, 0x45, 0x41, 0x31, 0x32, 0x2c, 0x20, 0x52, 0x45,
    0x4c, 0x45, 0x41, 0x53, 0x45, 0x20, 0x53, 0x4f, 0x46, 0x54, 0x57, 0x41, 0x52, 0x45, 0x20, 0x28,
    0x66, 0x63, 0x31, 0x29, 0x0a, 0x43, 0x6f, 0x70, 0x79, 0x72, 0x69, 0x67, 0x68, 0x74, 0x20, 0x28,
    0x63, 0x29, 0x20, 0x31, 0x39, 0x38, 0x36, 0x2d, 0x32, 0x30, 0x30, 0x38, 0x20, 0x62, 0x79, 0x20,
    0x63, 0x69, 0x73, 0x63, 0x6f, 0x20, 0x53, 0x79, 0x73, 0x74, 0x65, 0x6d, 0x73, 0x2c, 0x20, 0x49,
    0x6e, 0x63, 0x2e, 0x0a, 0x43, 0x6f, 0x6d, 0x70, 0x69, 0x6c, 0x65, 0x64, 0x20, 0x54, 0x75, 0x65,
    0x20, 0x30, 0x38, 0x2d, 0x4a, 0x75, 0x6c, 0x2d, 0x30, 0x38, 0x20, 0x30, 0x30, 0x3a, 0x30, 0x33,
    0x20, 0x62, 0x79, 0x20, 0x61, 0x6d, 0x76, 0x61, 0x72, 0x6d, 0x61, 0x00, 0x06, 0x00, 0x16, 0x63,
    0x69, 0x73, 0x63, 0x6f, 0x20, 0x57, 0x53, 0x2d, 0x43, 0x32, 0x39, 0x35, 0x30, 0x54, 0x2d, 0x32,
    0x34, 0x00, 0x08, 0x00, 0x24, 0x00, 0x00, 0x0c, 0x01, 0x12, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff,
    0xff, 0xff, 0x01, 0x02, 0x21, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x08, 0xa4, 0x94,
    0xcb, 0x00, 0xff, 0x00, 0x00, 0x00, 0x09, 0x00, 0x04, 0x00, 0x0a, 0x00, 0x06, 0x00, 0x01, 0x00,
    0x0b, 0x00, 0x05, 0x01, 0x00, 0x12, 0x00, 0x05, 0x00, 0x00, 0x13, 0x00, 0x05, 0x00, 0x00, 0x16,
    0x00, 0x11, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0xcc, 0x00, 0x04, 0x0a, 0x00, 0x00, 0xfe
  };
  const char dataset2[] __attribute__ ((aligned)) = { /* CDP packet data with checksum zeroed out */
              /*cksum:db37*/
    0x02, 0xb4, 0x00, 0x00, 0x00, 0x01, 0x00, 0x1a, 0x73, 0x77, 0x69, 0x74, 0x63, 0x68, 0x66, 0x6f,
    0x78, 0x32, 0x2e, 0x66, 0x6f, 0x78, 0x70, 0x61, 0x77, 0x73, 0x2e, 0x6e, 0x65, 0x74, 0x00, 0x02,
    0x00, 0x11, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0xcc, 0x00, 0x04, 0x0a, 0x00, 0x00, 0xfe, 0x00,
    0x03, 0x00, 0x13, 0x46, 0x61, 0x73, 0x74, 0x45, 0x74, 0x68, 0x65, 0x72, 0x6e, 0x65, 0x74, 0x30,
    0x2f, 0x31, 0x00, 0x04, 0x00, 0x08, 0x00, 0x00, 0x00, 0x28, 0x00, 0x05, 0x00, 0xe1, 0x43, 0x69,
    0x73, 0x63, 0x6f, 0x20, 0x49, 0x6e, 0x74, 0x65, 0x72, 0x6e, 0x65, 0x74, 0x77, 0x6f, 0x72, 0x6b,
    0x20, 0x4f, 0x70, 0x65, 0x72, 0x61, 0x74, 0x69, 0x6e, 0x67, 0x20, 0x53, 0x79, 0x73, 0x74, 0x65,
    0x6d, 0x20, 0x53, 0x6f, 0x66, 0x74, 0x77, 0x61, 0x72, 0x65, 0x20, 0x0a, 0x49, 0x4f, 0x53, 0x20,
    0x28, 0x74, 0x6d, 0x29, 0x20, 0x43, 0x32, 0x39, 0x35, 0x30, 0x20, 0x53, 0x6f, 0x66, 0x74, 0x77,
    0x61, 0x72, 0x65, 0x20, 0x28, 0x43, 0x32, 0x39, 0x35, 0x30, 0x2d, 0x49, 0x36, 0x4b, 0x32, 0x4c,
    0x32, 0x51, 0x34, 0x2d, 0x4d, 0x29, 0x2c, 0x20, 0x56, 0x65, 0x72, 0x73, 0x69, 0x6f, 0x6e, 0x20,
    0x31, 0x32, 0x2e, 0x31, 0x28, 0x32, 0x32, 0x29, 0x45, 0x41, 0x31, 0x32, 0x2c, 0x20, 0x52, 0x45,
    0x4c, 0x45, 0x41, 0x53, 0x45, 0x20, 0x53, 0x4f, 0x46, 0x54, 0x57, 0x41, 0x52, 0x45, 0x20, 0x28,
    0x66, 0x63, 0x31, 0x29, 0x0a, 0x43, 0x6f, 0x70, 0x79, 0x72, 0x69, 0x67, 0x68, 0x74, 0x20, 0x28,
    0x63, 0x29, 0x20, 0x31, 0x39, 0x38, 0x36, 0x2d, 0x32, 0x30, 0x30, 0x38, 0x20, 0x62, 0x79, 0x20,
    0x63, 0x69, 0x73, 0x63, 0x6f, 0x20, 0x53, 0x79, 0x73, 0x74, 0x65, 0x6d, 0x73, 0x2c, 0x20, 0x49,
    0x6e, 0x63, 0x2e, 0x0a, 0x43, 0x6f, 0x6d, 0x70, 0x69, 0x6c, 0x65, 0x64, 0x20, 0x54, 0x75, 0x65,
    0x20, 0x30, 0x38, 0x2d, 0x4a, 0x75, 0x6c, 0x2d, 0x30, 0x38, 0x20, 0x30, 0x30, 0x3a, 0x30, 0x33,
    0x20, 0x62, 0x79, 0x20, 0x61, 0x6d, 0x76, 0x61, 0x72, 0x6d, 0x61, 0x00, 0x06, 0x00, 0x16, 0x63,
    0x69, 0x73, 0x63, 0x6f, 0x20, 0x57, 0x53, 0x2d, 0x43, 0x32, 0x39, 0x35, 0x30, 0x54, 0x2d, 0x32,
    0x34, 0x00, 0x08, 0x00, 0x24, 0x00, 0x00, 0x0c, 0x01, 0x12, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff,
    0xff, 0xff, 0x01, 0x02, 0x21, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x08, 0xa4, 0x94,
    0xcb, 0x00, 0xff, 0x00, 0x00, 0x00, 0x09, 0x00, 0x04, 0x00, 0x0a, 0x00, 0x06, 0x00, 0x01, 0x00,
    0x0b, 0x00, 0x05, 0x01, 0x00, 0x12, 0x00, 0x05, 0x00, 0x00, 0x13, 0x00, 0x05, 0x00, 0x00, 0x16,
    0x00, 0x11, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0xcc, 0x00, 0x04, 0x0a, 0x00, 0x00, 0xfe
  };

  vector[0].ptr = dataset;
  vector[0].len = sizeof(dataset);
  vector[1].ptr = dataset2;
  vector[1].len = sizeof(dataset2);

  g_print("test packet data: %p length: %u\n", vector[0].ptr, vector[0].len);
  g_print("test packet data: %p length: %u\n", vector[1].ptr, vector[1].len);

  checksum = in_cksum(vector, 1);

  g_print("in_cksum                 %6d (0x%04x)\n", checksum, checksum);

  checksum = wireshark_cdp_checksum(vector[0].ptr, vector[0].len);

  g_print("WS CDP checksum          %6d (0x%04x)\n", checksum, checksum);

  checksum = real_cdp_checksum(vector[0].ptr, vector[0].len);

  g_print("CDP checksum             %6d (0x%04x)\n", checksum, checksum);

  checksum = real_cdp_checksum(vector[1].ptr, vector[1].len);

  g_print("CDP checksum(0'd)        %6d (0x%04x)\n", checksum, checksum);

  checksum = actual_cdp_checksum(vector[0].ptr, vector[0].len);

  g_print("Actual CDP checksum      %6d (0x%04x)\n", checksum, checksum);

  checksum = actual_cdp_checksum(vector[1].ptr, vector[1].len);

  g_print("Actual CDP checksum(0'd) %6d (0x%04x)\n", checksum, checksum);

  checksum = ios_cdp_checksum(vector[0].ptr, vector[0].len);

  g_print("IOS CDP checksum         %6d (0x%04x)\n", checksum, checksum);

  checksum = ios_cdp_checksum(vector[1].ptr, vector[1].len);

  g_print("IOS CDP checksum(0'd)    %6d (0x%04x)\n", checksum, checksum);
}
